# Contributing Guidelines

If you're thinking about contributing to this project, it's likely that you'll want to add support for parsing events from a currently unsupported website. The list of current adapters can be found in [`lib/url_2_events/adapters`](https://github.com/StartupWeekend/url_2_event/tree/master/lib/url_2_event/adapters).

## Adding a new website adapter

### Adapter class

First, get to know how each adapter works. To get started, create a file in the adapters folder called `website_name.rb`, replacing `website_name` with the name of the website you're creating. Create a class under the module `Url2Event` and have the class inherit from `Base`.

```ruby
module Url2Event
  class WebsiteName

  end
end
```

The major methods you'll need to implement are `to_event(response)`, `options`, `event_endpoint` and `self.is_parser_for?`. You'll also need to call `base_uri 'hostname'` in the class definition. Let's do this for an example website:

```ruby
module Url2Event
  class WebsiteName
    base_uri 'www.websitename.com'

    # this method should return a boolean, indicating whether
    # the `uri` given is supported by this adapter.
    def self.is_parser_for?(uri)
      !uri.hostname.match(%r{(?:www\.)?websitename\.com}).nil?
    end

    # return a hash, which will get passed to your http query.
    # this hash will get passed to `HTTParty`'s `get` method.
    def options
      @options = {
        query: {
          my_name: "is_good"
        }
      }
    end

    # return a string indicating the path that to be appended
    # to `base_uri` for this event. `@uri` is the URL that is passed
    # to your adapter.
    def event_endpoint
      "/api/#{@uri.path.split('/').last}"
    end

    # return an `Event` model based on the HTTP response you get
    # after hitting `event_endpoint`. Provide as many details as
    # possible for this event.
    def to_event(response)
      Event.new({
        title: response['title'],
        begin_at: response['start_time'],
        end_at: response['end_time'],
        location: response['address'],
        price: response['price'],
        description: response['description']
      })
    end
  end
end
```

Great! You're done setting up your adapter's class, and you're ready to move on to testing. We require that all adapters are tested, otherwise you're contribution won't be accepted.

### Adapter tests

To get started, create a new file called `spec/website_name_spec.rb`. Follow [proper rspec patterns](http://betterspecs.org/) while writing tests, and refer to other tests for examples. The goal is to write tests that go through example scenarios and prove that your adapter works the way it should.

To run tests, run `bundle exec guard` in the command line. As you write code, your tests will be run automatically.

```ruby
require 'spec_helper'

describe Url2Event::WebsiteName do
  let(:url) { URI.parse("http://www.websitename.com/events/187282242") }
  describe ".is_parser_for?" do
    it { expect(described_class.is_parser_for?(url)).to eql(true) }
  end

  # use `vcr` in any specs that use HTTP requests, otherwise your tests
  # will fail.
  describe "get_event", :vcr do
    let(:begin_at) { Time.parse("2014-07-29 18:00:00 +0000") }
    let(:end_at) { Time.parse("2014-07-29 21:00:00 +0000") }
    let(:title) { "Startup Grind Seattle Hosts Robert Scoble, Tech Geek (Rackspace)" }
    let(:parser) { described_class.new(url) }

    subject { parser.get_event }

    it { should be_instance_of(Event) }
    its(:title) { should eql(title) }
    its(:begin_at) { should eql(begin_at) }
    its(:end_at) { should eql(end_at) }
    its(:location) { should eql("Dice Cabana at SURF, 821 2nd Avenue #410, Seattle") }
    its(:price) { should be_nil }

    context "when the url is unparsable" do
      let(:unparsable) { URI.parse("http://www.websitename.com/Startup-Grind-Seattle") }
      let(:parser) { described_class.new(unparsable) }
      subject { parser.get_event }
      it { should raise_error(Url2Event::UnparsableSourceError) }
    end
  end
end
```

This is just an example, and you think for yourself about how your tests should work, and what scenarios you should test. At the least, you should test that:

- your adapter passes `self.is_adapter_for?` correctly
- `get_event` works for valid URLs
- `get_event` doesn't work for URLs that aren't parseable, like an 'about' page.

Make sure all tests pass when you run `rspec`. It's also a good idea to look in `coverage/index.html` to make sure your entire adapter's code is tested. If everything looks good, follow our Git instructions to open a pull request!

## Git Instructions

1. Fork it ( https://github.com/startupweekend/url_2_event/fork )
2. Create your feature branch (`git checkout -b my-new-feature`)
3. Commit your changes (`git commit -am 'Add some feature'`)
4. Push to the branch (`git push origin my-new-feature`)
5. Create a new Pull Request
